/*******************************************************************************
 * Quick access to the Components object.
 ******************************************************************************/
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
/*******************************************************************************
 * Component definitions.
 ******************************************************************************/
const SMS_CLASSNAME = "Netscape Sitemail Service";
const SMS_CLASSID = Components.ID("{759A7962-F7C2-479c-A18E-24292664852B}");
const SMS_CONTRACTID = "@netscape.com/sitemail/service;1";
/*******************************************************************************
 * Contract IDs.
 ******************************************************************************/
const PREF_CONTRACTID = "@mozilla.org/preferences-service;1";
const CAT_CONTRACTID = "@mozilla.org/categorymanager;1";
const OB_CONTRACTID = "@mozilla.org/observer-service;1";
const REQUEST_CONTRACTID = "@mozilla.org/xmlextras/xmlhttprequest;1";
const TIMER_CONTRACTID = "@mozilla.org/timer;1";
const BUNDLE_CONTRACTID = "@mozilla.org/intl/stringbundle;1";
const ALERT_CONTRACTID = "@mozilla.org/alerts-service;1";
const MEDIATOR_CONTRACTID = "@mozilla.org/appshell/window-mediator;1";
/*******************************************************************************
 * Observer topics
 ******************************************************************************/
const TOPIC_TIMER = "timer-callback";
const TOPIC_SITEMAIL = "netscape-sitemail";
const TOPIC_SHUTDOWN_APP = "quit-application";
const TOPIC_SHUTDOWN_XPCOM = "xpcom-shutdown";
const TOPIC_STARTUP_APP = "app-startup";
const TOPIC_STARTUP_UI = "final-ui-startup";
const TOPIC_PREF = "nsPref:changed";
const TOPIC_REQUEST = "http-on-examine-response";
const TOPIC_ALERT = "alertclickcallback";
/*******************************************************************************
 * Default values used across components.
 ******************************************************************************/
const DEFAULT_BRANCH = "extensions.netscape.sitemail.";
const DEFAULT_BUNDLE = "chrome://browser/locale/netscape-sitemail/sitemail.properties";
const DEFAULT_SEC = 1000;                                    // 1 second
const DEFAULT_MIN = 60*DEFAULT_SEC;                          // 1 minute
const DEFAULT_HOUR = 60*DEFAULT_MIN;                         // 1 hour
const DEFAULT_DAY = 24*DEFAULT_HOUR;                         // 1 day
const DEFAULT_TIMER = 100;                                   // 100 milliseconds
/*******************************************************************************
 * Sitemail checker values.
 ******************************************************************************/
const SMC_URL = "http://www.netscape.com/api/sitemail/";
const SMC_TIMER = 15*DEFAULT_SEC;                        // 15 seconds
const SMC_EXPIRES = 5*DEFAULT_MIN;                       // 5 minutes
const SMC_VAR = 10*DEFAULT_SEC;                          // 10 seconds
/*******************************************************************************
 * URL checker values.
 ******************************************************************************/
const URL_TIMER = 5*DEFAULT_TIMER;                        // 500 milliseconds
/*******************************************************************************
 * Alert manager values.
 ******************************************************************************/
const ALM_TITLE = "sitemail.module.alert.title";
const ALM_TEXT = "sitemail.module.alert.text.";
const ALM_IMAGE = "chrome://browser/skin/netscape-sitemail/sitemail-alert.png";
const ALM_WIN = "navigator:browser";
/*******************************************************************************
 * FAS service values.
 ******************************************************************************/
const SMS_ERROR = "http://www.mozilla.org/newlayout/xml/parsererror.xml";
const SMS_SIGNIN = "http://www.netscape.com/signin/?backurl=messages";
const SMS_URL = "http://www.netscape.com/member/%1/messages/";
const SMS_DATA_CHANGE = "change";
const SMS_DATA_SIGNIN = "signin";
const SMS_DATA_SIGNOUT = "signout";
/*******************************************************************************
 * Helper object for doing date time conversions.
 *
 * @param     value          A date value as a millisecond string.
 ******************************************************************************/
function DateUtil(value) {
  this.value = value;
}
DateUtil.prototype = {

  _value: "0",
  get value() { return this._value; },
  set value(val) { this._value = val; },
  
  get date() { return new Date(Number(this.value)); },
  set date(val) { this.value = String(val.getTime()); },  
  
  get time() { return this.date.getTime(); },
  set time(val) { this.date = new Date(val); },
  
  get string() { return this.date.toString(); },
  set string(val) { this.date = new Date(val); },

  get current() { return new Date(); },
  
  // date is less then the current date + a variance
  hasExpired: function du_hasExpired(variance) {
    return (this.time < (this.current.getTime() + variance)); 
  },
  
  // reset the data to current date + value
  reset: function du_reset(value) {
    var time = this.current.getTime() + value;
    var newDate = new Date(time);
    this.date = newDate;
  }
};
/*******************************************************************************
 * Object used to check the sitemail activity.  This will callback to the
 * sitemail service.
 *
 * @param     service        The sitemail service.
 ******************************************************************************/
function MailChecker(service) {
  this.smlService = service;
}
MailChecker.prototype = {

  _timer: null,
  get timer() { return this._timer; },
  set timer(val) { this._timer = val; },

  _firstTime: null,
  get firstTime() { return this._firstTime; },
  set firstTime(val) { this._firstTime = val; },
  
  _request: null,
  get request() { return this._request; },
  set request(val) { this._request = val; },

  _locked: null,
  get locked() { return this._locked; },
  set locked(val) { this._locked = val; },
  
  _expires: null,
  get expires() { return this._expires; },
  set expires(val) { this._expires = val; },
  
  _smlService: null,
  get smlService() { return this._smlService; },
  set smlService(val) { this._smlService = val; },
  
  _isPaused: null,
  get isPaused() { return this._isPaused; },
  set isPaused(val) { this._isPaused = val; },
    
  // nsISupports
  QueryInterface: function mlc_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
        
  // nsIObserver
  observe: function mlc_observe(subject, topic, data) { 
    if (topic == TOPIC_TIMER)
      this.fireTimer();
    else if (topic == TOPIC_PREF)
      this.reload();
  },
   
  needsUpdated: function mlc_needsUpdated(force) {
  
    // activity is being update we do not need another
    if (this.locked)
      return false;
    
    // force so we update
    if (force)
      return true;
      
    // feeds expired we need an update
    var util = new DateUtil(this.expires);
    if (util.hasExpired(SMC_VAR))
      return true;
      
    // no update needed
    return false;    
  },
   
  load: function mlc_load() {

    // add an observer to the pref branch
    this.smlService._branch.QueryInterface(Ci.nsIPrefBranch2).addObserver("", this, false);
    
    // start timer      
    this.expires = "0";
    this.isPaused = false;             
    this.firstTime = true;
    this.timer = Cc[TIMER_CONTRACTID].createInstance(Ci.nsITimer);
    this.timer.init(this, DEFAULT_TIMER, Ci.nsITimer.TYPE_ONE_SHOT);
  },

  unload: function mlc_unload() {

    // remove the pref observer
    this.smlService._branch.QueryInterface(Ci.nsIPrefBranch2).removeObserver("", this);
    
    // stop any request
    if (this.request)
      this.request.abort();
    
    // stop timer  
    this.timer.cancel();
    
    // remove variables
    this.timer = null;    
    this.firstTime = null;    
    this.request = null;  
    this.locked = null;    
    this.expires = null;      
    this.smlService = null;
    this.isPaused = null;
  },
  
  reload: function mlc_reload() {
    // stop request
    if (this.request)
      this.request.abort();
    this.locked = false;
    
    // request again
    this.requestUpdate(true);
  },
  
  pause: function mlc_pause() {
    this.timer.cancel();  
    this.isPaused = true;
  },
  
  resume: function mlc_resume() {
    if (!this.isPaused)
      return;
    this.timer.init(this, SMC_TIMER, Ci.nsITimer.TYPE_REPEATING_SLACK);  
    this.isPaused = false;
  },
  
  fireTimer: function mlc_fireTimer() {

    // change timer to a repeating timer if this is the first run
    if (this.firstTime) {
      this.firstTime = false;
      this.timer.init(this, SMC_TIMER, Ci.nsITimer.TYPE_REPEATING_SLACK);
    }
    
    // request an update
    this.requestUpdate(false);
  },
      
  requestUpdate: function mlc_requestUpdate(force) {
      
    // api does not need updating
    if (!this.needsUpdated(force))
      return;
           
    // create an XML HTTP request  
    var self = this;
    var url = SMC_URL;
    var request = Cc[REQUEST_CONTRACTID].createInstance(Ci.nsIXMLHttpRequest);
    request.open("GET", url, true);
    request.overrideMimeType("text/xml");
    request.onerror = function(event) { self.onError(event);    };
    request.onload = function(event) { self.onLoad(event);     };
    this.request = request;
    
    // send the request
    this.locked = true;
    this.request.send(null);
  },

  onLoad: function mlc_onLoad(event) {
  
    // callback the service
    var success = false;
    try {
      success = this.smlService._callback(this.request);  
    } catch(e) {
      success = false;
      Components.utils.reportError(e);
    }
    
    // release the request
    this.locked = false;
    this.request = null;
    
    // set the expires date
    var util = new DateUtil("0");
    var value = SMC_EXPIRES;
    if (success)
      value = this.smlService._branch.getIntPref("frequency")*DEFAULT_MIN;
    util.reset(value);
    this.expires = util.value;
  },

  onError: function mlc_onError(event) {   
    // release the request 
    this.locked = false;
    this.request = null;
    
    // set the expires date
    var util = new DateUtil("0");
    util.reset(SMC_EXPIRES);
    this.expires = util.value;
    
    // make sure we are not paused  
    this.resume();  
  }
};
/*******************************************************************************
 * Object used to check URLs for signin/signout.  This will callback to the
 * sitemail service.
 *
 * @param     service        The fas service.
 ******************************************************************************/
function URLChecker(service) {
  this.smlService = service;
}
URLChecker.prototype = {

  _timer: null,
  get timer() { return this._timer; },
  set timer(val) { this._timer = val; },

  _firstTime: null,
  get firstTime() { return this._firstTime; },
  set firstTime(val) { this._firstTime = val; },
  
  _smlService: null,
  get smlService() { return this._smlService; },
  set smlService(val) { this._smlService = val; },
  
  _queue: null,
  get queue() { return this._queue; },
  set queue(val) { this._queue = val; },

  _homePage: null,
  get homePage() { return this._homePage; },
  set homePage(val) { this._homePage = val; },
  
  _signinPage: null,
  get signinPage() { return this._signinPage; },
  set signinPage(val) { this._signinPage = val; },
  
  _signoutPage: null,
  get signoutPage() { return this._signoutPage; },
  set signoutPage(val) { this._signoutPage = val; },
  
  _messagePage: null,
  get messagePage() { return this._messagePage; },
  set messagePage(val) { this._messagePage = val; },
  
  // nsISupports
  QueryInterface: function url_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
        
  // nsIObserver
  observe: function url_observe(subject, topic, data) { 
    if (topic != TOPIC_TIMER)
      return;
    this.fireTimer();
  },
   
  load: function url_load() {
    
    // setup the regexp
    var pattern = "^https?:\/\/www\.netscape\.com\/signin";    
    this.signinPage = new RegExp(pattern, "i");
    pattern = "^https?:\/\/www\.netscape\.com\/signout";
    this.signoutPage = new RegExp(pattern, "i");
    pattern = "^https?:\/\/www\.netscape\.com$"; 
    this.homePage = new RegExp(pattern, "i");
    pattern = "^https?:\/\/www\.netscape\.com\/member/[^/]+/messages";  
    this.messagePage = new RegExp(pattern, "i");
    
    // setup the timer     
    this.queue = [];
    this.firstTime = true;
    this.timer = Cc[TIMER_CONTRACTID].createInstance(Ci.nsITimer);
    this.timer.init(this, DEFAULT_TIMER, Ci.nsITimer.TYPE_ONE_SHOT);
  },

  unload: function url_unload() {
  
    // stop the timer
    this.timer.cancel();
    
    // remove variables
    this.timer = null;    
    this.firstTime = null;    
    this.smlService = null;
    this.queue = null;
    this.homePage = null;
    this.signinPage = null;
    this.signoutPage = null;
    this.messagePage = null;
  },

  
  fireTimer: function url_fireTimer() {

    // change timer to a repeating timer if this is the first run
    if (this.firstTime) {
      this.firstTime = false;
      this.timer.init(this, URL_TIMER, Ci.nsITimer.TYPE_REPEATING_SLACK);
    }
    
    // read through the queue
    this.checkQueue();
  },
      
  checkQueue: function url_checkQueue() {
    while (this.queue.length) {
      var url = this.queue.shift();
      this.checkURL(url);    
    }
  },

  addURL: function url_addURL(url) {
    this.queue.push(url);
  },
  
  checkURL: function url_checkURL(url) {   

    // user is signing out
    if (url.match(this.signoutPage))
      this.smlService._clearData(true);
   
    // user is signing in
    else if (url.match(this.signinPage))
      this.smlService.checkMail();
  
    // user is reading message
    else if (url.match(this.messagePage))
      this.smlService.checkMail();
   
    // user is hitting the homepage
    else if (url.match(this.homePage))
      this.smlService.checkMail(); 
  } 
};
/*******************************************************************************
 * Object used to display an alert notification.
 *
 * @param     service        The fas service.
 ******************************************************************************/
function AlertManager(service) {
  this.smlService = service;
}
AlertManager.prototype = {

  _timer: null,
  get timer() { return this._timer; },
  set timer(val) { this._timer = val; },

  _smlService: null,
  get smlService() { return this._smlService; },
  set smlService(val) { this._smlService = val; },
  
  _bundle: null,
  get bundle() { return this._bundle; },
  set bundle(val) { this._bundle = val; },

  _totalCount: null,
  get totalCount() { return this._totalCount; },
  set totalCount(val) { this._totalCount = val; },
  
  _prevCount: null,
  get prevCount() { return this._prevCount; },
  set prevCount(val) { this._prevCount = val; },
    
  // nsISupports
  QueryInterface: function alm_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
        
  // nsIObserver
  observe: function alm_observe(subject, topic, data) { 
    if (topic == TOPIC_ALERT)
      this.open(subject.QueryInterface(Ci.nsIPropertyBag));
    else if (topic == TOPIC_TIMER)
      this.fireTimer();
  },
   
  load: function alm_load() {
    // setup count
    this.totalCount = 0;
    this.prevCount = 0;
    
    // setup string bundle
    var sbService = Cc[BUNDLE_CONTRACTID].getService(Ci.nsIStringBundleService);
    this.bundle = sbService.createBundle(DEFAULT_BUNDLE); 
    
    // setup the timer
    this.timer = Cc[TIMER_CONTRACTID].createInstance(Ci.nsITimer);   
  },

  unload: function alm_unload() {

    // stop the timer
    this.timer.cancel();
    
    // remove the variables
    this.timer = null;
    this.smlService = null;
    this.bundle = null;
    this.totalCount = null;
    this.prevCount = null;    
  },

  
  fireTimer: function alm_fireTimer() {
    
    // no alerts
    if (this.totalCount < 1)
      return;
    
    // count has not changed
    if (this.totalCount == this.prevCount)
      return;
          
    // do nothing if we do not have the alert service
    var showAlert = (ALERT_CONTRACTID in Cc);
    if (!showAlert)
      return;
      
    // do nothing if not enabled
    showAlert = this.smlService._branch.getBoolPref("showAlert");
    if (!showAlert)
      return;
    
    // show the alert
    var service = Cc[ALERT_CONTRACTID].getService(Ci.nsIAlertsService);
    var title = this.bundle.GetStringFromName(ALM_TITLE);
    var text;
    if (this.totalCount == 1)
      text = ALM_TEXT + "single";   
    else
      text = ALM_TEXT + "plural";
    text = this.bundle.formatStringFromName(text, [this.totalCount], 1);
    service.showAlertNotification(ALM_IMAGE, title, text, true, null, this);
  },

  showAlert: function alm_showAlert(count) {
    this.prevCount = this.totalCount;  
    this.totalCount = count;
    this.timer.init(this, DEFAULT_TIMER, Ci.nsITimer.TYPE_ONE_SHOT);
  },
  
  open: function alm_open(bag) {
    var mediator = Cc[MEDIATOR_CONTRACTID].getService(Ci.nsIWindowMediator);
    var win = mediator.getMostRecentWindow(ALM_WIN);
    if (!win)
      return;
    var e = {
      shiftKey: bag.getProperty("shiftKey"),
      ctrlKey: bag.getProperty("ctrlKey"),
      metaKey: bag.getProperty("metaKey"),
      altKey: bag.getProperty("altKey"),
      button: bag.getProperty("button")    
    };
    win.Sitemail.open(e);  
  }
};
/*******************************************************************************
 * The main sitemail interface.  This interface takes care of updating the 
 * mail list.  It notifies the front-end when updates take place.
 * 
 * @version   1.0
 ******************************************************************************/
function SitemailService() {}
SitemailService.prototype = {
  _userName: null,
  _totalCount: null,
  _URL: null,
  _obService: null,
  _mailChecker: null,
  _urlChecker: null,
  _alertManager: null,
  _branch: null,
        
  // nsISupports
  QueryInterface: function sms_QI(iid) {
    if (iid.equals(Ci.nsISitemailService) ||
        iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  // nsIObserver
  observe: function sms_observe(subject, topic, data) {
    switch (topic) {
          
    case TOPIC_STARTUP_APP:
      this._load();
      break;
    
    case TOPIC_STARTUP_UI:
      this._loadDelayed();
      break;
    
    case TOPIC_SHUTDOWN_APP:
      this._unload();
      break;
              
    case TOPIC_SHUTDOWN_XPCOM:
      this._unloadDelayed();
      break;
    
    case TOPIC_REQUEST:
      var url = subject.QueryInterface(Ci.nsIHttpChannel).originalURI.spec;    
      this._urlChecker.addURL(url);
      break;
      
    default:
      return;
    }
  },
    
  get isSignedIn() { return (this._userName !== null); },
  get userName() { return this._userName; },
  get hasMail() { return (this._totalCount > 0); },
  get totalCount() { return this._totalCount; },
  get URL() { return this._URL; },
  

  checkMail: function sms_checkMail() {
    this._mailChecker.requestUpdate(true);
  },
        
  _load: function sms__load() {
    
    // get the pref branch
    this._branch = Cc[PREF_CONTRACTID].getService(Ci.nsIPrefService).
                   getBranch(DEFAULT_BRANCH);
    
    // get the observer service and add ourself as an observer               
    this._obService = Cc[OB_CONTRACTID].getService(Ci.nsIObserverService);
    this._obService.addObserver(this, TOPIC_STARTUP_UI, false);
    this._obService.addObserver(this, TOPIC_SHUTDOWN_APP, false);
    this._obService.addObserver(this, TOPIC_SHUTDOWN_XPCOM, false);
    this._obService.addObserver(this, TOPIC_REQUEST, false);

    // setup the variables
    this._clearData(false);
    this._mailChecker = new MailChecker(this);    
    this._urlChecker = new URLChecker(this);
    this._alertManager = new AlertManager(this);
  },
  
  _loadDelayed: function sms__loadDelayed() {
    this._mailChecker.load();
    this._urlChecker.load();
    this._alertManager.load();
  },
  
  _unload: function sms__unload() {
    this._mailChecker.unload();
    this._urlChecker.unload();
    this._alertManager.unload();    
  },
  
  _unloadDelayed: function sms__unloadDelayed() {
         
    // remove observers
    this._obService.removeObserver(this, TOPIC_STARTUP_UI);
    this._obService.removeObserver(this, TOPIC_SHUTDOWN_APP);    
    this._obService.removeObserver(this, TOPIC_SHUTDOWN_XPCOM);
    this._obService.removeObserver(this, TOPIC_REQUEST);
    
    // remove variables
    this._userName = null;
    this._totalCount = null;
    this._URL = null;
    this._obService = null;    
    this._mailChecker = null;
    this._urlChecker = null;
    this._alertManager = null;
    this._branch = null;   
  },
  
  _callback: function sms__callback(request) {
    var wasSignedIn = this.isSignedIn;

    // on request error unpause and return error
    var xml = request.responseXML;
    if (!xml || xml.documentElement.namespaceURI == SMS_ERROR ||
       (request.status != 200 && request.status !== 0)) {
      this._mailChecker.resume();
      return false;
    }  
    
    // reset the values and return if signed out
	var member = xml.getElementsByTagName("member")[0];
	var signedIn = parseInt(member.getAttribute("loggedIn"));
    this._clearData(!signedIn);
	if (!signedIn)
      return true;

    // resume the timer
    this._mailChecker.resume();

    // setup the url
    this._userName = member.getElementsByTagName("alias")[0].firstChild.nodeValue;    
    var url = SMS_URL.replace("%1", this.userName);
    this._URL = url;

    // setup the counts
    this._totalCount = parseInt(xml.getElementsByTagName("sitemail")[0].getElementsByTagName("messagecount")[0].firstChild.nodeValue);
    this._alertManager.showAlert(this._totalCount);
    
    // notify the front-end we changed and return success
    data = (!wasSignedIn) ? SMS_DATA_SIGNIN : SMS_DATA_CHANGE;
    this._notify(data);
    return true;
  },
  
  _clearData: function sms__clearData(notify) {
    var wasSignedIn = this.isSignedIn;
    
    this._userName = null;
    this._totalCount = 0;
    this._URL = SMS_SIGNIN;  
  
    if (notify) {
      this._mailChecker.pause();
      var data = (wasSignedIn) ? SMS_DATA_SIGNOUT : SMS_DATA_CHANGE;
      this._notify(data);
    }
  },
  
  _notify: function sms__notify(data) {
    try {
      this._obService.notifyObservers(this, TOPIC_SITEMAIL, data);
    } catch(e) {
      Components.utils.reportError(e);
    }
  }    
};
/*******************************************************************************
 * Generic object used as a factory based on a passed in constructor.
 ******************************************************************************/
function GenericFactory(ctor) {
  this._ctor = ctor;
}  
GenericFactory.prototype = {
  _ctor: null,
  
  // nsISupports
  QueryInterface: function gf_QI(iid) {
    if (iid.equals(Ci.nsIFactory) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  // nsIFactory
  createInstance: function gf_createInstance(outer, iid) {
    if (outer !== null)
      throw Cr.NS_ERROR_NO_AGGREGATION;
    return (new this._ctor()).QueryInterface(iid);
  },

  lockFactory: function gf_lockFactory(loc) {
    throw Cr.NS_ERROR_NOT_IMPLEMENTED;
  }
};
/*******************************************************************************
 * Object implementing the nsIModule interface.  This is used to retrieve 
 * the different components and services the module implements.
 ******************************************************************************/
var Module = {

  // nsISupports
  QueryInterface: function mod_QI(iid) {
    if (iid.equals(Ci.nsIModule) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE;
  },
  
  // nsIModule
  getClassObject: function mod_getClassObject(cm, cid, iid) {
    if (!iid.equals(Ci.nsIFactory))
      throw Cr.NS_ERROR_NOT_IMPLEMENTED;
    
    if (cid.equals(SMS_CLASSID))
      return new GenericFactory(SitemailService);
          
    throw Cr.NS_ERROR_NO_INTERFACE;
  },

  registerSelf: function mod_registerSelf(cm, file, location, type) {
    var cr = cm.QueryInterface(Ci.nsIComponentRegistrar);  
    cr.registerFactoryLocation(SMS_CLASSID, SMS_CLASSNAME, SMS_CONTRACTID,
                               file, location, type);   
                                
    var catMgr = Components.classes[CAT_CONTRACTID].
                 getService(Ci.nsICategoryManager);
    catMgr.addCategoryEntry(TOPIC_STARTUP_APP, SMS_CLASSNAME, "service," + 
                            SMS_CONTRACTID, true, true);                                                                                         
  },

  unregisterSelf: function mod_unregisterSelf(cm, location, type) {
    var cr = cm.QueryInterface(Ci.nsIComponentRegistrar);
    cr.unregisterFactoryLocation(SMS_CLASSID, location);
                                
    var catMgr = Components.classes[CAT_CONTRACTID].
                 getService(Ci.nsICategoryManager); 
    catMgr.deleteCategoryEntry(TOPIC_STARTUP_APP, "service," + 
                               SMS_CONTRACTID, true);
  },

  canUnload: function mod_canUnload(cm) { return true; }
};
/*******************************************************************************
 * Entry point used to retrieve the module object.
 ******************************************************************************/
function NSGetModule(cm, file) { return Module; }